"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var figures = require("figures");
var chalk = require("chalk");
var util_1 = require("./util");
var MigrationPrinter = (function () {
    function MigrationPrinter(out) {
        this.out = out;
    }
    MigrationPrinter.prototype.printMessages = function (migrationMessages) {
        var _this = this;
        migrationMessages.forEach(function (migrationMessage, index) {
            var actionType = _this.getMigrationActionType(migrationMessage.description);
            var symbol = _this.getSymbolForMigrationActionType(actionType);
            var description = util_1.makePartsEnclodesByCharacterBold(migrationMessage.description, "`");
            var outputMessage = (index > 0 ? "  |" : "") + "\n  | (" + symbol + ")  " + description + "\n";
            _this.out.log(outputMessage);
            migrationMessage.subDescriptions.forEach(function (subMessage, index2) {
                var actionType2 = _this.getMigrationActionType(subMessage.description);
                var symbol2 = _this.getSymbolForMigrationActionType(actionType2);
                var outputMessage2 = util_1.makePartsEnclodesByCharacterBold(subMessage.description, "`");
                var lastLine = index2 === migrationMessage.subDescriptions.length - 1;
                var endSymbol = lastLine ? '└' : '├';
                _this.out.log("  " + endSymbol + "\u2500\u2500 (" + symbol2 + ")  " + outputMessage2 + "\n");
            });
        });
    };
    MigrationPrinter.prototype.printErrors = function (errors) {
        var _this = this;
        var indentation = spaces(2);
        errors.forEach(function (error) {
            var outputMessage = util_1.makePartsEnclodesByCharacterBold(error.description, "`");
            _this.out.log("" + indentation + chalk.red(figures.cross) + " " + outputMessage + "\n");
        });
    };
    MigrationPrinter.prototype.getMigrationActionType = function (message) {
        if (message.indexOf('create') >= 0) {
            return 'create';
        }
        else if (message.indexOf('update') >= 0) {
            return 'update';
        }
        else if (message.indexOf('delete') >= 0 || message.indexOf('remove') >= 0) {
            return 'delete';
        }
        return 'unknown';
    };
    MigrationPrinter.prototype.getSymbolForMigrationActionType = function (type) {
        switch (type) {
            case 'create': return '+';
            case 'delete': return '-';
            case 'update': return '*';
            case 'unknown': return '?';
        }
    };
    return MigrationPrinter;
}());
exports.MigrationPrinter = MigrationPrinter;
var spaces = function (n) { return Array(n + 1).join(' '); };
//# sourceMappingURL=migration.js.map